@@ -14,7 +14,7 @@
 #
 ##############################################################################
 """
-test.py [-abcCdDfgGhLmprtTuv] [modfilter [testfilter]]
+test.py [-abcCdDfgGhLmpqrtTuv] [modfilter [testfilter]]
 
 Find and run tests written using the unittest module.
 
@@ -104,6 +104,9 @@ named .testinfo, it will not be searched for tests.  Really.)
     Keep running the selected tests in a loop.  You may experience
     memory leakage.
 
+--nowarnings
+    Install a filter to suppress warnings emitted by code.
+
 -t
     Time the individual tests and print a list of the top 50, sorted from
     longest to shortest.
@@ -142,10 +145,11 @@ named .testinfo, it will not be searched for tests.  Really.)
     KCacheGrind sources to create data suitable for analysis by KCacheGrind.
 
 -v
+-q
     Verbose output.  With one -v, unittest prints a dot (".") for each
     test run.  With -vv, unittest prints the name of each test (for
-    some definition of "name" ...).  With no -v, unittest is silent
-    until the end of the run, except when errors occur.
+    some definition of "name" ...).  With no -v (or with -q), unittest is
+    silent until the end of the run, except when errors occur.
 
 -u
 -m
@@ -606,6 +610,8 @@ class TrackRefs:
 def runner(files, test_filter, debug):
     runner = ImmediateTestRunner(verbosity=VERBOSE, debug=debug,
                                  progress=progress)
+    result = runner.result
+    
     suite = unittest.TestSuite()
     for file in files:
         s = get_suite(file, runner.result)
@@ -622,6 +628,8 @@ def runner(files, test_filter, debug):
                 print "Wrote timing data to", timesfn
         if timetests:
             r.print_times(sys.stdout, timetests)
+        numbad = len(result.failures) + len(result.errors)
+        return numbad
     except:
         if debugger:
             pdb.post_mortem(sys.exc_info()[2])
@@ -703,7 +711,7 @@ def main(module_filter, test_filter, libdir):
             rc = sys.gettotalrefcount()
             track = TrackRefs()
         while True:
-            runner(files, test_filter, debug)
+            numbad = runner(files, test_filter, debug)
             gc.collect()
             if gc.garbage:
                 print "GARBAGE:", len(gc.garbage), gc.garbage
@@ -714,8 +722,9 @@ def main(module_filter, test_filter, libdir):
                 print "totalrefcount=%-8d change=%-6d" % (rc, rc - prev)
                 track.update()
     else:
-        runner(files, test_filter, debug)
+        numbad = runner(files, test_filter, debug)
 
+    return numbad
 
 def configure_logging():
     """Initialize the logging module."""
@@ -759,6 +768,7 @@ def process_args(argv=None):
     global test_dir
     global config_file
     global import_testing
+    global no_warnings
 
     if argv is None:
         argv = sys.argv
@@ -789,11 +799,12 @@ def process_args(argv=None):
     test_dir = None
     config_file = None
     import_testing = False
+    no_warnings = False
 
     try:
-        opts, args = getopt.getopt(argv[1:], "a:bcC:dDfg:G:hLmprtTuv",
+        opts, args = getopt.getopt(argv[1:], "a:bcC:dDfg:G:hLmpqrtTuv",
                                    ["all", "help", "libdir=", "times=",
-                                    "keepbytecode", "dir=", 
+                                    "keepbytecode", "nowarnings", "dir=",
                                     "config-file=", "import-testing",
                                     "coverage", "profile", "hotshot"])
     except getopt.error, msg:
@@ -860,6 +871,8 @@ def process_args(argv=None):
             GUI = 1
         elif k == "-v":
             VERBOSE += 1
+        elif k == "-q":
+            VERBOSE = 0
         elif k == "--times":
             try:
                 timetests = int(v)
@@ -872,6 +885,12 @@ def process_args(argv=None):
             config_file = v
         elif k == '--import-testing':
             import_testing = True
+        elif k == '--nowarnings':
+            no_warnings = True
+
+    if no_warnings:
+        import warnings
+        warnings.simplefilter('ignore', Warning, append=1)
 
     if gcthresh is not None:
         if gcthresh == 0:
