@@ -12,7 +12,7 @@
 ##############################################################################
 """Property management
 
-$Id: PropertyManager.py 30356 2005-05-15 15:56:18Z yuppie $
+$Id: PropertyManager.py 68072 2006-05-10 08:24:46Z shh $
 """
 
 from cgi import escape
@@ -117,7 +117,8 @@ class PropertyManager(ExtensionClass.Base, ZDOM.ElementWithAttributes):
                                )),
         ('Access contents information',
          ('hasProperty', 'propertyIds', 'propertyValues','propertyItems',
-          'getProperty', 'getPropertyType', 'propertyMap', ''),
+          'getProperty', 'getPropertyType', 'propertyMap', 'propertyLabel',
+          'propdict', 'valid_property_id', ''),
          ('Anonymous', 'Manager'),
          ),
         )
