@@ -15,9 +15,9 @@
 Zope object encapsulating a Page Template.
 """
 
-__version__='$Revision: 1.44.44.5 $'[11:-2]
+__version__='$Revision: 1.48 $'[11:-2]
 
-import os, AccessControl, Acquisition, sys
+import os, AccessControl, Acquisition, sys, types
 from types import StringType
 from Globals import DTMLFile, ImageFile, MessageDialog, package_home
 from zLOG import LOG, ERROR, INFO
@@ -45,6 +45,23 @@ try:
 except ImportError:
     SUPPORTS_WEBDAV_LOCKS = 0
 
+
+
+class Src(Acquisition.Explicit):
+    " "
+
+    PUT = document_src = Acquisition.Acquired
+    index_html = None
+
+    def __before_publishing_traverse__(self, ob, request):
+        if getattr(request, '_hacked_path', 0):
+            request._hacked_path = 0
+
+    def __call__(self, REQUEST, RESPONSE):
+        " "
+        return self.document_src(REQUEST)
+
+
 class ZopePageTemplate(Script, PageTemplate, Historical, Cacheable,
                        Traversable, PropertyManager):
     "Zope wrapper for Page Template using TAL, TALES, and METAL"
@@ -94,7 +111,7 @@ class ZopePageTemplate(Script, PageTemplate, Historical, Cacheable,
     security.declareProtected('View management screens',
       'pt_editForm', 'manage_main', 'read',
       'ZScriptHTML_tryForm', 'PrincipiaSearchSource',
-      'document_src', 'source.html', 'source.xml')
+      'document_src', 'source_dot_xml')
 
     security.declareProtected('FTP access',
       'manage_FTPstat','manage_FTPget','manage_FTPlist')
@@ -104,6 +121,8 @@ class ZopePageTemplate(Script, PageTemplate, Historical, Cacheable,
     pt_editForm._owner = None
     manage = manage_main = pt_editForm
 
+    source_dot_xml = Src()
+
     security.declareProtected('Change Page Templates',
       'pt_editAction', 'pt_setTitle', 'pt_edit',
       'pt_upload', 'pt_changePrefs')
@@ -115,6 +134,7 @@ class ZopePageTemplate(Script, PageTemplate, Historical, Cacheable,
         self.pt_setTitle(title)
         self.pt_edit(text, content_type)
         REQUEST.set('text', self.read()) # May not equal 'text'!
+        REQUEST.set('title', self.title)
         message = "Saved changes."
         if getattr(self, '_v_warnings', None):
             message = ("<strong>Warning:</strong> <i>%s</i>"
@@ -122,9 +142,18 @@ class ZopePageTemplate(Script, PageTemplate, Historical, Cacheable,
         return self.pt_editForm(manage_tabs_message=message)
 
     def pt_setTitle(self, title):
-        self._setPropValue('title', str(title))
+        charset = getattr(self, 'management_page_charset', None)
+        if type(title) == types.StringType and charset:
+            try:
+                title.decode('us-ascii')
+                title = str(title)
+            except UnicodeError:
+                title = unicode(title, charset)
+        elif type(title) != types.UnicodeType:
+            title = str(title)
+        self._setPropValue('title', title)
 
-    def pt_upload(self, REQUEST, file=''):
+    def pt_upload(self, REQUEST, file='', charset=None):
         """Replace the document with the text in file."""
         if SUPPORTS_WEBDAV_LOCKS and self.wl_isLocked():
             raise ResourceLockedError, "File is locked via WebDAV"
@@ -132,7 +161,12 @@ class ZopePageTemplate(Script, PageTemplate, Historical, Cacheable,
         if type(file) is not StringType:
             if not file: raise ValueError, 'File not specified'
             file = file.read()
-
+        if charset:
+            try:
+                unicode(file, 'us-ascii')
+                file = str(file)
+            except UnicodeDecodeError:
+                file = unicode(file, charset)
         self.write(file)
         message = 'Saved changes.'
         return self.pt_editForm(manage_tabs_message=message)
@@ -294,24 +328,8 @@ class ZopePageTemplate(Script, PageTemplate, Historical, Cacheable,
         def wl_isLocked(self):
             return 0
 
-
-class Src(Acquisition.Explicit):
-    " "
-
-    PUT = document_src = Acquisition.Acquired
-    index_html = None
-
-    def __before_publishing_traverse__(self, ob, request):
-        if getattr(request, '_hacked_path', 0):
-            request._hacked_path = 0
-
-    def __call__(self, REQUEST, RESPONSE):
-        " "
-        return self.document_src(REQUEST)
-
-d = ZopePageTemplate.__dict__
-d['source.xml'] = d['source.html'] = Src()
-
+setattr(ZopePageTemplate, 'source.xml',  ZopePageTemplate.source_dot_xml)
+setattr(ZopePageTemplate, 'source.html', ZopePageTemplate.source_dot_xml)
 
 # Product registration and Add support
 manage_addPageTemplateForm = PageTemplateFile(
@@ -334,11 +352,13 @@ def manage_addPageTemplate(self, id, title=None, text=None,
         file = REQUEST.form.get('file')
         headers = getattr(file, 'headers', None)
         if headers is None or not file.filename:
-            zpt = ZopePageTemplate(id, text)
+            zpt = ZopePageTemplate(id, text) # collector 596
         else:
             zpt = ZopePageTemplate(id, file, headers.get('content_type'))
 
         self._setObject(id, zpt)
+
+        # collector 596
         if title:
             ob = getattr(self, id)
             ob.pt_setTitle(title)
