@@ -7,13 +7,24 @@
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-import os, sys, unittest, zLOG
+"""KeywordIndex unit tests.
+
+$Id: testKeywordIndex.py 31005 2005-07-05 13:38:18Z yuppie $
+"""
+
+import unittest
+import Testing
+import Zope2
+Zope2.startup()
+
+import zLOG
 
 from Products.PluginIndexes.KeywordIndex.KeywordIndex import KeywordIndex
 
+
 class Dummy:
 
     def __init__( self, foo ):
@@ -35,6 +46,7 @@ def sortedUnique(seq):
     unique.sort()
     return unique
 
+
 class TestKeywordIndex( unittest.TestCase ):
     """
         Test KeywordIndex objects.
@@ -102,6 +114,16 @@ class TestKeywordIndex( unittest.TestCase ):
         for k, v in expectedValues:
             assert k in result
 
+    def test_z3interfaces(self):
+        from Products.PluginIndexes.interfaces import IPluggableIndex
+        from Products.PluginIndexes.interfaces import ISortIndex
+        from Products.PluginIndexes.interfaces import IUniqueValueIndex
+        from zope.interface.verify import verifyClass
+
+        verifyClass(IPluggableIndex, KeywordIndex)
+        verifyClass(ISortIndex, KeywordIndex)
+        verifyClass(IUniqueValueIndex, KeywordIndex)
+
     def testAddObjectWOKeywords(self):
 
         self._catch_log_errors()
@@ -114,6 +136,7 @@ class TestKeywordIndex( unittest.TestCase ):
     def testEmpty( self ):
         assert len( self._index ) == 0
         assert len( self._index.referencedObjects() ) == 0
+        self.assertEqual(self._index.numObjects(), 0)
 
         assert self._index.getEntryForObject( 1234 ) is None
         assert ( self._index.getEntryForObject( 1234, self._marker )
@@ -134,13 +157,14 @@ class TestKeywordIndex( unittest.TestCase ):
         self._populateIndex()
         values = self._values
 
-        self.assertEqual(self._index.numObjects(), len(self._index))
+        #assert len( self._index ) == len( values )
         assert len( self._index.referencedObjects() ) == len( values )
 
         assert self._index.getEntryForObject( 1234 ) is None
         assert ( self._index.getEntryForObject( 1234, self._marker )
                   is self._marker )
         self._index.unindex_object( 1234 ) # nothrow
+        self.assertEqual(self._index.indexSize(), len( values )-1)
 
         for k, v in values:
             entry = self._index.getEntryForObject( k )
@@ -210,7 +234,7 @@ class TestKeywordIndex( unittest.TestCase ):
         #
         record[ 'foo' ][ 'range' ] = 'min:max'
         self._checkApply( record, self._values[6:7] )
-        
+
     def testDuplicateKeywords(self):
         self._catch_log_errors()
         try:
@@ -219,14 +243,38 @@ class TestKeywordIndex( unittest.TestCase ):
         finally:
             self._ignore_log_errors()
 
+    def testCollectorIssue889(self) :
+        # Test that collector issue 889 is solved
+        values = self._values
+        nonexistent = 'foo-bar-baz'
+        self._populateIndex()
+        # make sure key is not indexed
+        result = self._index._index.get(nonexistent, self._marker)
+        assert result is self._marker
+        # patched _apply_index now works as expected
+        record = {'foo' : { 'query'    : [nonexistent]
+                          , 'operator' : 'and'}
+                 }
+        self._checkApply(record, [])
+        record = {'foo' : { 'query'    : [nonexistent, 'a']
+                          , 'operator' : 'and'}
+                 }
+        # and does not break anything
+        self._checkApply(record, [])
+        record = {'foo' : { 'query'    : ['d']
+                          , 'operator' : 'and'}
+                 }
+        self._checkApply(record, values[4:5])
+        record = {'foo' : { 'query'    : ['a', 'e']
+                          , 'operator' : 'and'}
+                 }
+        self._checkApply(record, values[5:7])
+
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest( unittest.makeSuite( TestKeywordIndex ) )
     return suite
-    
-def main():
-    unittest.main(defaultTest='test_suite')
 
 if __name__ == '__main__':
-    main()
-
+    unittest.main(defaultTest='test_suite')
