@@ -17,11 +17,12 @@ This product provides support for Script objects containing restricted
 Python code.
 """
 
-__version__='$Revision: 1.43.6.1 $'[11:-2]
+__version__='$Revision: 1.56 $'[11:-2]
 
-import sys, os, traceback, re, marshal
+import sys, os, traceback, re, marshal, new
 from Globals import DTMLFile, MessageDialog, package_home
 import AccessControl, OFS, RestrictedPython
+from Acquisition import aq_parent
 from OFS.SimpleItem import SimpleItem
 from DateTime.DateTime import DateTime
 from urllib import quote
@@ -31,9 +32,11 @@ from Shared.DC.Scripts.Script import Script, BindingsUI, defaultBindings
 from AccessControl import getSecurityManager
 from OFS.History import Historical, html_diff
 from OFS.Cache import Cacheable
-from AccessControl import full_write_guard, safe_builtins
-from AccessControl.ZopeGuards import guarded_getattr, guarded_getitem
+from AccessControl.ZopeGuards import get_safe_globals, guarded_getattr
+from AccessControl.requestmethod import postonly
 from zLOG import LOG, ERROR, INFO, PROBLEM
+from zExceptions import Forbidden
+import Globals
 
 # Track the Python bytecode version
 import imp
@@ -115,7 +118,7 @@ class PythonScript(Script, Historical, Cacheable):
     security.declareProtected('View management screens',
       'ZPythonScriptHTML_editForm', 'manage_main', 'read',
       'ZScriptHTML_tryForm', 'PrincipiaSearchSource',
-      'document_src', 'params', 'body')
+      'document_src', 'params', 'body', 'get_filepath')
 
     ZPythonScriptHTML_editForm = DTMLFile('www/pyScriptEdit', globals())
     manage = manage_main = ZPythonScriptHTML_editForm
@@ -164,20 +167,26 @@ class PythonScript(Script, Historical, Cacheable):
                                                manage_tabs_message=message)
 
     def ZPythonScriptHTML_changePrefs(self, REQUEST, height=None, width=None,
-                                      dtpref_cols='50', dtpref_rows='20'):
+                                      dtpref_cols="100%", dtpref_rows="20"):
         """Change editing preferences."""
-        szchh = {'Taller': 1, 'Shorter': -1, None: 0}
-        szchw = {'Wider': 5, 'Narrower': -5, None: 0}
-        try: rows = int(height)
-        except: rows = max(1, int(dtpref_rows) + szchh.get(height, 0))
-        try: cols = int(width)
-        except: cols = max(40, int(dtpref_cols) + szchw.get(width, 0))
-        e = (DateTime('GMT') + 365).rfc822()
-        setc = REQUEST['RESPONSE'].setCookie
-        setc('dtpref_rows', str(rows), path='/', expires=e)
-        setc('dtpref_cols', str(cols), path='/', expires=e)
-        REQUEST.form.update({'dtpref_cols': cols, 'dtpref_rows': rows})
-        return apply(self.manage_main, (self, REQUEST), REQUEST.form)
+        dr = {"Taller":5, "Shorter":-5}.get(height, 0)
+        dc = {"Wider":5, "Narrower":-5}.get(width, 0)
+        if isinstance(height, int): dtpref_rows = height
+        if isinstance(width, int) or \
+           isinstance(width, str) and width.endswith('%'):
+            dtpref_cols = width
+        rows = str(max(1, int(dtpref_rows) + dr))
+        cols = str(dtpref_cols)
+        if cols.endswith('%'):
+           cols = str(min(100, max(25, int(cols[:-1]) + dc))) + '%'
+        else:
+           cols = str(max(35, int(cols) + dc))
+        e = (DateTime("GMT") + 365).rfc822()
+        setCookie = REQUEST["RESPONSE"].setCookie
+        setCookie("dtpref_rows", rows, path='/', expires=e)
+        setCookie("dtpref_cols", cols, path='/', expires=e)
+        REQUEST.other.update({"dtpref_cols":cols, "dtpref_rows":rows})
+        return self.manage_main(self, REQUEST)
 
     def ZScriptHTML_tryParams(self):
         """Parameters to test the script with."""
@@ -186,7 +195,7 @@ class PythonScript(Script, Historical, Cacheable):
 
             name = name.strip()
             if name and name[0] != '*' and re.match('\w',name):
-                param_names.append(name.split('=', 1)[0])
+                param_names.append(name.split('=', 1)[0].strip())
         return param_names
 
     def manage_historyCompare(self, rev1, rev2, REQUEST,
@@ -210,21 +219,24 @@ class PythonScript(Script, Historical, Cacheable):
             self._compile()
             self._v_change = 1
         elif self._code is None:
-            self._v_f = None
+            self._v_ft = self._v_f = None
         else:
             self._newfun(marshal.loads(self._code))
 
-    def _compiler(self, *args):
-        return RestrictedPython.compile_restricted_function(*args)
+    def _compiler(self, *args, **kw):
+        return RestrictedPython.compile_restricted_function(*args, **kw)
+
     def _compile(self):
+        bind_names = self.getBindingAssignments().getAssignedNamesInOrder()
         r = self._compiler(self._params, self._body or 'pass',
-                           self.id, self.meta_type)
+                           self.id, self.meta_type,
+                           globalize=bind_names)
         code = r[0]
         errors = r[1]
         self.warnings = tuple(r[2])
         if errors:
             self._code = None
-            self._v_f = None
+            self._v_ft = self._v_f = None
             self._setFuncSignature((), (), 0)
             # Fix up syntax errors.
             filestring = '  File "<string>",'
@@ -246,24 +258,26 @@ class PythonScript(Script, Historical, Cacheable):
         self._v_change = 0
 
     def _newfun(self, code):
-        g = {'__debug__': __debug__,
-             '__builtins__': safe_builtins,
-             '_getattr_': guarded_getattr,
-             '_getitem_': guarded_getitem,
-             '_write_': full_write_guard,
-             '_print_': RestrictedPython.PrintCollector,
-             }
+        g = get_safe_globals()
+        g['_getattr_'] = guarded_getattr
+        g['__debug__'] = __debug__
+        g['__name__'] = None
+
         l = {}
         exec code in g, l
         self._v_f = f = l.values()[0]
+        self._v_ft = (f.func_code, g, f.func_defaults or ())
         return f
 
     def _makeFunction(self, dummy=0): # CMFCore.FSPythonScript uses dummy arg.
         self.ZCacheable_invalidate()
         self._compile()
+        if not (aq_parent(self) is None or hasattr(self, '_filepath')):
+            # It needs a _filepath, and has an acquisition wrapper.
+            self._filepath = self.get_filepath()
 
     def _editedBindings(self):
-        if getattr(self, '_v_f', None) is not None:
+        if getattr(self, '_v_ft', None) is not None:
             self._makeFunction()
 
     def _exec(self, bound_names, args, kw):
@@ -279,7 +293,7 @@ class PythonScript(Script, Historical, Cacheable):
             asgns = self.getBindingAssignments()
             name_context = asgns.getAssignedName('name_context', None)
             if name_context:
-                keyset[name_context] = self.aq_parent.getPhysicalPath()
+                keyset[name_context] = aq_parent(self).getPhysicalPath()
             name_subpath = asgns.getAssignedName('name_subpath', None)
             if name_subpath:
                 keyset[name_subpath] = self._getTraverseSubpath()
@@ -292,33 +306,33 @@ class PythonScript(Script, Historical, Cacheable):
 
         #__traceback_info__ = bound_names, args, kw, self.func_defaults
 
-        f = self._v_f
-        if f is None:
+        ft = self._v_ft
+        if ft is None:
             __traceback_supplement__ = (
                 PythonScriptTracebackSupplement, self)
             raise RuntimeError, '%s %s has errors.' % (self.meta_type, self.id)
 
+        fcode, g, fadefs = ft
+        g = g.copy()
         if bound_names is not None:
-            # XXX This causes the whole acquisition chain
-            # to be held by self._v_f.  I think we really should
-            # use new.function() instead, similar to
-            # CMFCore.FSPythonScript.  new.function() takes
-            # about 8 microseconds on a 1 GHz Athlon. - Shane
-            f.func_globals.update(bound_names)
-        f.func_globals['__traceback_supplement__'] = (
+            g.update(bound_names)
+        g['__traceback_supplement__'] = (
             PythonScriptTracebackSupplement, self, -1)
+        g['__file__'] = getattr(self, '_filepath', None) or self.get_filepath()
+        f = new.function(fcode, g, None, fadefs)
 
-        # Execute the function in a new security context.
-        security=getSecurityManager()
-        security.addContext(self)
-        try:
-            result = apply(f, args, kw)
-            if keyset is not None:
-                # Store the result in the cache.
-                self.ZCacheable_set(result, keywords=keyset)
-            return result
-        finally:
-            security.removeContext(self)
+        result = f(*args, **kw)
+        if keyset is not None:
+            # Store the result in the cache.
+            self.ZCacheable_set(result, keywords=keyset)
+        return result
+
+    def manage_afterAdd(self, item, container):
+        if item is self:
+            self._filepath = self.get_filepath()
+
+    def get_filepath(self):
+        return self.meta_type + ':' + '/'.join(self.getPhysicalPath())
 
     def manage_haveProxy(self,r): return r in self._proxy_roles
 
@@ -328,7 +342,7 @@ class PythonScript(Script, Historical, Cacheable):
         user = getSecurityManager().getUser()
         if user is not None and user.allowed(self, roles):
             return
-        raise 'Forbidden', ('You are not authorized to change <em>%s</em> '
+        raise Forbidden, ('You are not authorized to change <em>%s</em> '
             'because you do not have proxy roles.\n<!--%s, %s-->'
             % (self.id, user, roles))
 
@@ -346,6 +360,7 @@ class PythonScript(Script, Historical, Cacheable):
                     title  ='Success!',
                     message='Your changes have been saved',
                     action ='manage_main')
+    manage_proxy = postonly(manage_proxy)
 
     security.declareProtected('Change Python Scripts',
       'PUT', 'manage_FTPput', 'write',
@@ -481,6 +496,8 @@ class PythonScript(Script, Historical, Cacheable):
         return self.read()
 
 
+Globals.InitializeClass(PythonScript)
+
 class PythonScriptTracebackSupplement:
     """Implementation of ITracebackSupplement"""
     def __init__(self, script, line=0):
