@@ -25,6 +25,30 @@ else:
     if not here:
         here = os.getcwd()
 
+class WarningInterceptor:
+
+    _old_stderr = None
+    _our_stderr_stream = None
+
+    def _trap_warning_output( self ):
+
+        if self._old_stderr is not None:
+            return
+
+        import sys
+        from StringIO import StringIO
+
+        self._old_stderr = sys.stderr
+        self._our_stderr_stream = sys.stderr = StringIO()
+
+    def _free_warning_output( self ):
+
+        if self._old_stderr is None:
+            return
+
+        import sys
+        sys.stderr = self._old_stderr
+
 # Test Classes
 
 def readf(name):
@@ -40,7 +64,6 @@ class VerifiedPythonScript(PythonScript):
 
 class PythonScriptTestBase(unittest.TestCase):
     def setUp(self):
-        # Turn deprecation warnings into exceptions
         newSecurityManager(None, None)
 
     def tearDown(self):
@@ -203,12 +226,12 @@ class TestPythonScriptErrors(PythonScriptTestBase):
         self.assertPSRaises(ImportError, body="import mmap")
 
     def testAttributeAssignment(self):
-        # It's illegal to assign to attributes of anything except
-        # list or dict.
+        # It's illegal to assign to attributes of anything that
+        # doesn't has enabling security declared.
+        # Classes (and their instances) defined by restricted code
+        # are an exception -- they are fully readable and writable.
         cases = [("import string", "string"),
-                 ("class Spam: pass", "Spam"),
                  ("def f(): pass", "f"),
-                 ("class Spam: pass\nspam = Spam()", "spam"),
                  ]
         assigns = ["%s.splat = 'spam'",
                    "setattr(%s, '_getattr_', lambda x, y: True)",
@@ -220,7 +243,15 @@ class TestPythonScriptErrors(PythonScriptTestBase):
                 f = self._newPS(defn + "\n" + asn % name)
                 self.assertRaises(TypeError, f)
 
-class TestPythonScriptGlobals(PythonScriptTestBase):
+class TestPythonScriptGlobals(PythonScriptTestBase, WarningInterceptor):
+
+    def setUp(self):
+        PythonScriptTestBase.setUp(self)
+
+    def tearDown(self):
+        self._free_warning_output()
+        PythonScriptTestBase.tearDown(self)
+
     def _exec(self, script, bound_names=None, args=None, kws=None):
         if args is None:
             args = ()
@@ -237,11 +268,15 @@ class TestPythonScriptGlobals(PythonScriptTestBase):
 
     def test__name__(self):
         f = self._filePS('class.__name__')
-        self.assertEqual(f(), ('?.foo', "'string'"))
+        self.assertEqual(f(), ("'foo'>", "'string'"))
 
     def test_filepath(self):
+        # This test is meant to raise a deprecation warning.
+        # It used to fail mysteriously instead.
+        self._trap_warning_output()
         f = self._filePS('filepath')
         self.assertEqual(f(), [0])
+        self._free_warning_output()
 
 def test_suite():
     suite = unittest.TestSuite()
