@@ -4,7 +4,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -13,9 +13,7 @@
 ##############################################################################
 """ZODB-defined exceptions
 
-$Id: POSException.py,v 1.20.4.5 2004/09/09 18:18:19 tim_one Exp $"""
-
-from types import StringType, DictType
+$Id: POSException.py 29676 2005-03-24 23:04:52Z tim_one $"""
 
 from ZODB.utils import oid_repr, readable_tid_repr
 
@@ -35,6 +33,16 @@ class POSKeyError(KeyError, POSError):
 class TransactionError(POSError):
     """An error occured due to normal transaction processing."""
 
+class TransactionFailedError(POSError):
+    """Cannot perform an operation on a transaction that previously failed.
+
+    An attempt was made to commit a transaction, or to join a transaction,
+    but this transaction previously raised an exception during an attempt
+    to commit it.  The transaction must be explicitly aborted, either by
+    invoking abort() on the transaction, or begin() on its transaction
+    manager.
+    """
+
 class ConflictError(TransactionError):
     """Two transactions tried to modify the same object at once.
 
@@ -52,13 +60,17 @@ class ConflictError(TransactionError):
         related to conflict.  The first is the revision of object that
         is in conflict, the currently committed serial.  The second is
         the revision the current transaction read when it started.
+      data : string
+        The database record that failed to commit, used to put the
+        class name in the error message.
 
     The caller should pass either object or oid as a keyword argument,
     but not both of them.  If object is passed, it should be a
     persistent object with an _p_oid attribute.
     """
 
-    def __init__(self, message=None, object=None, oid=None, serials=None):
+    def __init__(self, message=None, object=None, oid=None, serials=None,
+                 data=None):
         if message is None:
             self.message = "database conflict error"
         else:
@@ -76,6 +88,14 @@ class ConflictError(TransactionError):
             assert self.oid is None
             self.oid = oid
 
+        if data is not None:
+            # avoid circular import chain
+            from ZODB.utils import get_pickle_metadata
+            self.class_name = "%s.%s" % get_pickle_metadata(data)
+##        else:
+##            if message != "data read conflict error":
+##                raise RuntimeError
+
         self.serials = serials
 
     def __str__(self):
@@ -116,22 +136,76 @@ class ReadConflictError(ConflictError):
     An attempt was made to read an object that has changed in another
     transaction (eg. another thread or process).
     """
-    def __init__(self, object):
-        ConflictError.__init__(self, message="database read conflict error",
-                               object=object)
-        self.jar = object._p_jar
-
-    def ignore(self):
-        self.jar._ignore_conflict(self.oid)
+    def __init__(self, message=None, object=None, serials=None):
+        if message is None:
+            message = "database read conflict error"
+        ConflictError.__init__(self, message=message, object=object,
+                               serials=serials)
 
 class BTreesConflictError(ConflictError):
-    """A special subclass for BTrees conflict errors.
+    """A special subclass for BTrees conflict errors."""
 
-    These return an undocumented four-tuple.
-    """
-    def __init__(self, *btree_args):
-        ConflictError.__init__(self, message="BTrees conflict error")
-        self.btree = btree_args
+    msgs = [# 0; i2 or i3 bucket split; positions are all -1
+            'Conflicting bucket split',
+
+            # 1; keys the same, but i2 and i3 values differ, and both values
+            # differ from i1's value
+            'Conflicting changes',
+
+            # 2; i1's value changed in i2, but key+value deleted in i3
+            'Conflicting delete and change',
+
+            # 3; i1's value changed in i3, but key+value deleted in i2
+            'Conflicting delete and change',
+
+            # 4; i1 and i2 both added the same key, or both deleted the
+            # same key
+            'Conflicting inserts or deletes',
+
+            # 5;  i2 and i3 both deleted the same key
+            'Conflicting deletes',
+
+            # 6; i2 and i3 both added the same key
+            'Conflicting inserts',
+
+            # 7; i2 and i3 both deleted the same key, or i2 changed the value
+            # associated with a key and i3 deleted that key
+            'Conflicting deletes, or delete and change',
+
+            # 8; i2 and i3 both deleted the same key, or i3 changed the value
+            # associated with a key and i2 deleted that key
+            'Conflicting deletes, or delete and change',
+
+            # 9; i2 and i3 both deleted the same key
+            'Conflicting deletes',
+
+            # 10; i2 and i3 deleted all the keys, and didn't insert any,
+            # leaving an empty bucket; conflict resolution doesn't have
+            # enough info to unlink an empty bucket from its containing
+            # BTree correctly
+            'Empty bucket from deleting all keys',
+
+            # 11; conflicting changes in an internal BTree node
+            'Conflicting changes in an internal BTree node',
+
+            # 12; i2 or i3 was empty
+            'Empty bucket in a transaction',
+            ]
+
+    def __init__(self, p1, p2, p3, reason):
+        self.p1 = p1
+        self.p2 = p2
+        self.p3 = p3
+        self.reason = reason
+
+    def __repr__(self):
+        return "BTreesConflictError(%d, %d, %d, %d)" % (self.p1,
+                                                        self.p2,
+                                                        self.p3,
+                                                        self.reason)
+    def __str__(self):
+        return "BTrees conflict error at %d/%d/%d: %s" % (
+            self.p1, self.p2, self.p3, self.msgs[self.reason])
 
 class DanglingReferenceError(TransactionError):
     """An object has a persistent reference to a missing object.
@@ -210,7 +284,7 @@ class ExportError(POSError):
     """An export file doesn't have the right format."""
 
 class Unsupported(POSError):
-    """An feature that is unsupported bt the storage was used."""
+    """A feature was used that is not supported by the storage."""
 
 class InvalidObjectReference(POSError):
     """An object contains an invalid reference to another object.
@@ -221,7 +295,7 @@ class InvalidObjectReference(POSError):
 
     o A reference to an object in a different database connection.
 
-    XXX The exception ought to have a member that is the invalid object.
+    TODO:  The exception ought to have a member that is the invalid object.
     """
 
 class ConnectionStateError(POSError):
@@ -233,4 +307,3 @@ class ConnectionStateError(POSError):
       still joined to a transaction (for example, a transaction is in
       progress, with uncommitted modifications in the connection).
     """
-
