@@ -11,9 +11,9 @@
 #
 ##############################################################################
 
-__version__='$Revision: 1.90.2.6 $'[11:-2]
+__version__='$Revision: 1.96 $'[11:-2]
 
-import re, sys, os,  urllib, time, random, cgi, codecs
+import re, sys, os, time, random, codecs
 from types import StringType, UnicodeType
 from BaseRequest import BaseRequest
 from HTTPResponse import HTTPResponse
@@ -25,6 +25,9 @@ from TaintedString import TaintedString
 from maybe_lock import allocate_lock
 xmlrpc=None # Placeholder for module that we'll import if we have to.
 
+# This may get overwritten during configuration
+default_encoding = 'iso-8859-15'
+
 isCGI_NAME = {
         'SERVER_SOFTWARE' : 1,
         'SERVER_NAME' : 1,
@@ -205,7 +208,6 @@ class HTTPRequest(BaseRequest):
             If the URL makes no sense in light of the current virtual
             hosting context, a ValueError is raised."""
         other = self.other
-        bad_server_url = 0
         path = filter(None, URL.split( '/'))
 
         if URL.find( '://') >= 0:
@@ -214,7 +216,6 @@ class HTTPRequest(BaseRequest):
         # Check the path against BASEPATH1
         vhbase = self._script
         vhbl = len(vhbase)
-        bad_basepath = 0
         if path[:vhbl] == vhbase:
             path = path[vhbl:]
         else:
@@ -386,7 +387,7 @@ class HTTPRequest(BaseRequest):
         if not hasattr(fs,'list') or fs.list is None:
             # Hm, maybe it's an XML-RPC
             if (fs.headers.has_key('content-type') and
-                fs.headers['content-type'] == 'text/xml' and
+                'text/xml' in fs.headers['content-type'] and
                 method == 'POST'):
                 # Ye haaa, XML-RPC!
                 global xmlrpc
@@ -404,7 +405,7 @@ class HTTPRequest(BaseRequest):
             CGI_name=isCGI_NAME
             defaults={}
             tainteddefaults={}
-            converter=seqf=None
+            converter=None
 
             for item in fslist:
 
@@ -453,10 +454,8 @@ class HTTPRequest(BaseRequest):
                             converter_type = type_name
                             flags=flags|CONVERTED
                         elif type_name == 'list':
-                            seqf=list
                             flags=flags|SEQUENCE
                         elif type_name == 'tuple':
-                            seqf=tuple
                             tuple_items[key]=1
                             flags=flags|SEQUENCE
                         elif (type_name == 'method' or type_name == 'action'):
@@ -526,7 +525,7 @@ class HTTPRequest(BaseRequest):
                                 if hasattr(converter,'convert_unicode'):
                                     item = converter.convert_unicode(item)
                                 else:
-                                    item = converter(item.encode('latin1'))
+                                    item = converter(item.encode(default_encoding))
                             else:
                                 item=converter(item)
 
@@ -1040,7 +1039,7 @@ class HTTPRequest(BaseRequest):
         except: rsp.exception()
         if object is None:
             req.close()
-            raise rsp.errmsg, sys.exc_info()[1]
+            raise sys.exc_info()[0], rsp.errmsg
 
         # The traversal machinery may return a "default object"
         # like an index_html document. This is not appropriate
@@ -1237,6 +1236,8 @@ class HTTPRequest(BaseRequest):
             if (isCGI_NAME(key) or key[:5] == 'HTTP_') and (not hide_key(key)):
                 keys[key] = 1
 
+        # Cache URLN and BASEN in self.other.
+        # This relies on a side effect of has_key.
         n=0
         while 1:
             n=n+1
@@ -1332,12 +1333,28 @@ class HTTPRequest(BaseRequest):
             if auth[:6].lower() == 'basic ':
                 if base64 is None: import base64
                 [name,password] = \
-                    base64.decodestring(auth.split()[-1]).split(':')
+                    base64.decodestring(auth.split()[-1]).split(':', 1)
                 return name, password
 
     def taintWrapper(self, enabled=TAINTING_ENABLED):
         return enabled and TaintRequestWrapper(self) or self
 
+    # Used by Five
+
+    def getPresentationSkin(self):
+        """see zope.component.interfaces.IPresentationRequest"""
+        return getattr(self, '_presentation_skin', None)
+
+    def setPresentationSkin(self, skin):
+        """see zope.publisher.interfaces.IPublicationRequest"""
+        self._presentation_skin = skin
+
+    def shiftNameToApplication(self):
+        """see zope.publisher.interfaces.http.IVirtualHostRequest"""
+        # this is needed for ++skin++
+
+    def getURL(self):
+        return self.URL
 
 class TaintRequestWrapper:
     def __init__(self, req):
@@ -1440,6 +1457,8 @@ def parse_cookie(text,
                     '([\x00- ]*([^\x00- ;,="]+)="([^"]*)"([\x00- ]*[;,])?[\x00- ]*)'),
                  parmre=re.compile(
                     '([\x00- ]*([^\x00- ;,="]+)=([^\x00- ;,"]*)([\x00- ]*[;,])?[\x00- ]*)'),
+                 paramlessre=re.compile(
+                    '([\x00- ]*([^\x00- ;,="]+)[\x00- ]*[;,][\x00- ]*)'),
 
                  acquire=parse_cookie_lock.acquire,
                  release=parse_cookie_lock.release,
@@ -1471,7 +1490,15 @@ def parse_cookie(text,
                 value = mo_p.group(3)
 
             else:
-                return result
+                # Broken Cookie without = nor value.
+ 		broken_p = paramlessre.match(text)
+ 		if broken_p:
+ 		    l = len(broken_p.group(1))
+ 		    name = broken_p.group(2)
+ 		    value = ''
+
+                else:
+                    return result
 
     finally: release()
 
