@@ -13,14 +13,12 @@
 
 """WebDAV support - resource objects.
 
-$Id: Resource.py,v 1.55.10.3 2003/11/17 22:34:22 tseaver Exp $
+$Id: Resource.py 65606 2006-02-28 19:55:33Z sidnei $
 """
 
-import os
 import sys
 import mimetypes
-import time
-from urllib import quote, unquote
+from urllib import unquote
 
 import Globals
 import ExtensionClass
@@ -28,7 +26,7 @@ from Acquisition import aq_base
 from AccessControl import getSecurityManager
 from ZPublisher.HTTPRangeSupport import HTTPRangeInterface
 from zExceptions import Unauthorized, Forbidden
-from zExceptions import BadRequest, MethodNotAllowed 
+from zExceptions import BadRequest, MethodNotAllowed, NotFound
 
 from WriteLockInterface import WriteLockInterface
 import Lockable
@@ -67,14 +65,12 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         # Init expected HTTP 1.1 / WebDAV headers which are not
         # currently set by the base response object automagically.
         #
-        # Note we set an borg-specific header for ie5 :( Also, we
-        # sniff for a ZServer response object, because we don't
+        # We sniff for a ZServer response object, because we don't
         # want to write duplicate headers (since ZS writes Date
         # and Connection itself).
         if not hasattr(response, '_server_version'):
             response.setHeader('Connection', 'close')
             response.setHeader('Date', rfc1123_date(), 1)
-        response.setHeader('MS-Author-Via', 'DAV')
 
         # HTTP Range support
         if HTTPRangeInterface.isImplementedBy(self):
@@ -258,6 +254,13 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         self.dav__init(REQUEST, RESPONSE)
         cmd=davcmds.PropFind(REQUEST)
         result=cmd.apply(self)
+        # work around MSIE DAV bug for creation and modified date
+        if (REQUEST.get_header('User-Agent') ==
+            'Microsoft Data Access Internet Publishing Provider DAV 1.1'):
+            result = result.replace('<n:getlastmodified xmlns:n="DAV:">',
+                                    '<n:getlastmodified xmlns:n="DAV:" xmlns:b="urn:uuid:c2f41010-65b3-11d1-a29f-00aa00c14882/" b:dt="dateTime.rfc1123">')
+            result = result.replace('<n:creationdate xmlns:n="DAV:">',
+                                    '<n:creationdate xmlns:n="DAV:" xmlns:b="urn:uuid:c2f41010-65b3-11d1-a29f-00aa00c14882/" b:dt="dateTime.tz">')
         RESPONSE.setStatus(207)
         RESPONSE.setHeader('Content-Type', 'text/xml; charset="utf-8"')
         RESPONSE.setBody(result)
@@ -317,7 +320,6 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
             raise BadRequest, 'Invalid Destination header'
 
         name = path.pop()
-        parent_path = '/'.join(path)
 
         oflag=REQUEST.get_header('Overwrite', 'F').upper()
         if not oflag in ('T', 'F'):
@@ -371,13 +373,10 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
             else:
                 raise Locked, 'Destination is locked.'
 
-        ob=self._getCopy(parent)
-        ob.manage_afterClone(ob)
-        # We remove any locks from the copied object because webdav clients
-        # don't track the lock status and the lock token for copied resources
-        ob.wl_clearLocks()
-
+        self._notifyOfCopyTo(parent, op=0)
+        ob = self._getCopy(parent)
         ob._setId(name)
+
         if depth=='0' and isDavCollection(ob):
             for id in ob.objectIds():
                 ob._delObject(id)
@@ -386,6 +385,12 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
             self.dav__validate(object, 'DELETE', REQUEST)
             parent._delObject(name)
         parent._setObject(name, ob)
+        ob = parent._getOb(name)
+        ob._postCopy(parent, op=0)
+        ob.manage_afterClone(ob)
+        # We remove any locks from the copied object because webdav clients
+        # don't track the lock status and the lock token for copied resources
+        ob.wl_clearLocks()
         RESPONSE.setStatus(existing and 204 or 201)
         if not existing:
             RESPONSE.setHeader('Location', dest)
@@ -478,7 +483,12 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
                 raise PreconditionFailed, 'Source is locked and no '\
                       'condition was passed in.'
 
-        ob=aq_base(self._getCopy(parent))
+        # try to make ownership explicit so that it gets carried
+        # along to the new location if needed.
+        self.manage_changeOwnershipType(explicit=1)
+
+        self._notifyOfCopyTo(parent, op=1)
+        ob = aq_base(self._getCopy(parent))
         self.aq_parent._delObject(absattr(self.id))
         ob._setId(name)
         if existing:
@@ -486,6 +496,12 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
             self.dav__validate(object, 'DELETE', REQUEST)
             parent._delObject(name)
         parent._setObject(name, ob)
+        ob = parent._getOb(name)
+        ob._postCopy(parent, op=1)
+
+        # try to make ownership implicit if possible
+        ob.manage_changeOwnershipType(explicit=0)
+
         RESPONSE.setStatus(existing and 204 or 201)
         if not existing:
             RESPONSE.setHeader('Location', dest)
@@ -558,7 +574,6 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         """Remove an existing lock on a resource."""
         self.dav__init(REQUEST, RESPONSE)
         security = getSecurityManager()
-        user = security.getUser()
         token = REQUEST.get_header('Lock-Token', '')
         url = REQUEST['URL']
         token = tokenFinder(token)
